@@ -1,4 +1,4 @@
-from typing import List
+from typing import Any, List
 
 from ..service import BleakGATTService
 from .characteristic import BleakGATTCharacteristicBlueZDBus
@@ -8,7 +8,7 @@ from .utils import extract_service_handle_from_path
 class BleakGATTServiceBlueZDBus(BleakGATTService):
     """GATT Service implementation for the BlueZ DBus backend"""
 
-    def __init__(self, obj, path):
+    def __init__(self, obj: Any, path: str):
         super().__init__(obj)
         self.__characteristics = []
         self.__path = path
@@ -29,7 +29,9 @@ class BleakGATTServiceBlueZDBus(BleakGATTService):
         """List of characteristics for this service"""
         return self.__characteristics
 
-    def add_characteristic(self, characteristic: BleakGATTCharacteristicBlueZDBus):
+    def add_characteristic(
+        self, characteristic: BleakGATTCharacteristicBlueZDBus
+    ) -> None:
         """Add a :py:class:`~BleakGATTCharacteristicBlueZDBus` to the service.
 
         Should not be used by end user, but rather by `bleak` itself.
@@ -37,6 +39,6 @@ class BleakGATTServiceBlueZDBus(BleakGATTService):
         self.__characteristics.append(characteristic)
 
     @property
-    def path(self):
+    def path(self) -> str:
         """The DBus path. Mostly needed by `bleak`, not by end user"""
         return self.__path
