@@ -8,11 +8,18 @@ Created on June, 25 2019 by kevincar <kevincarrolldavis@gmail.com>
 
 import asyncio
 import logging
+import sys
 import threading
-from typing import Any, Callable, Dict, Optional
+from typing import Any, Callable, Dict, List, Optional
+
+if sys.version_info < (3, 11):
+    from async_timeout import timeout as async_timeout
+else:
+    from asyncio import timeout as async_timeout
 
 import objc
 from CoreBluetooth import (
+    CBUUID,
     CBCentralManager,
     CBManagerStatePoweredOff,
     CBManagerStatePoweredOn,
@@ -21,9 +28,9 @@ from CoreBluetooth import (
     CBManagerStateUnknown,
     CBManagerStateUnsupported,
     CBPeripheral,
-    CBUUID,
 )
 from Foundation import (
+    NSUUID,
     NSArray,
     NSDictionary,
     NSError,
@@ -32,12 +39,10 @@ from Foundation import (
     NSNumber,
     NSObject,
     NSString,
-    NSUUID,
 )
-from libdispatch import dispatch_queue_create, DISPATCH_QUEUE_SERIAL
+from libdispatch import DISPATCH_QUEUE_SERIAL, dispatch_queue_create
 
-from bleak.backends.corebluetooth.device import BLEDeviceCoreBluetooth
-from bleak.exc import BleakError
+from ...exc import BleakError
 
 logger = logging.getLogger(__name__)
 CBCentralManagerDelegate = objc.protocolNamed("CBCentralManagerDelegate")
@@ -61,8 +66,6 @@ class CentralManagerDelegate(NSObject):
         self.event_loop = asyncio.get_running_loop()
         self._connect_futures: Dict[NSUUID, asyncio.Future] = {}
 
-        self.devices: Dict[str, BLEDeviceCoreBluetooth] = {}
-
         self.callbacks: Dict[
             int, Callable[[CBPeripheral, Dict[str, Any], int], None]
         ] = {}
@@ -84,6 +87,9 @@ class CentralManagerDelegate(NSObject):
         if self.central_manager.state() == CBManagerStateUnsupported:
             raise BleakError("BLE is unsupported")
 
+        if self.central_manager.state() == CBManagerStateUnauthorized:
+            raise BleakError("BLE is not authorized - check macOS privacy settings")
+
         if self.central_manager.state() != CBManagerStatePoweredOn:
             raise BleakError("Bluetooth device is turned off")
 
@@ -97,17 +103,20 @@ class CentralManagerDelegate(NSObject):
 
         return self
 
-    def __del__(self):
+    def __del__(self) -> None:
         if objc.macos_available(10, 13):
-            self.central_manager.removeObserver_forKeyPath_(self, "isScanning")
+            try:
+                self.central_manager.removeObserver_forKeyPath_(self, "isScanning")
+            except IndexError:
+                # If self.init() raised an exception before calling
+                # addObserver_forKeyPath_options_context_, attempting
+                # to remove the observer will fail with IndexError
+                pass
 
     # User defined functions
 
     @objc.python_method
-    async def start_scan(self, service_uuids) -> None:
-        # remove old
-        self.devices = {}
-
+    async def start_scan(self, service_uuids: Optional[List[str]]) -> None:
         service_uuids = (
             NSArray.alloc().initWithArray_(
                 list(map(CBUUID.UUIDWithString_, service_uuids))
@@ -149,7 +158,7 @@ class CentralManagerDelegate(NSObject):
         self,
         peripheral: CBPeripheral,
         disconnect_callback: DisconnectCallback,
-        timeout=10.0,
+        timeout: float = 10.0,
     ) -> None:
         try:
             self._disconnect_callbacks[peripheral.identifier()] = disconnect_callback
@@ -158,7 +167,8 @@ class CentralManagerDelegate(NSObject):
             self._connect_futures[peripheral.identifier()] = future
             try:
                 self.central_manager.connectPeripheral_options_(peripheral, None)
-                await asyncio.wait_for(future, timeout=timeout)
+                async with async_timeout(timeout):
+                    await future
             finally:
                 del self._connect_futures[peripheral.identifier()]
 
@@ -250,29 +260,17 @@ class CentralManagerDelegate(NSObject):
 
         uuid_string = peripheral.identifier().UUIDString()
 
-        if uuid_string in self.devices:
-            device = self.devices[uuid_string]
-            # It could be the device did not have a name previously but now it does.
-            if peripheral.name():
-                device.name = peripheral.name()
-        else:
-            address = uuid_string
-            name = peripheral.name() or None
-            details = peripheral
-            device = BLEDeviceCoreBluetooth(address, name, details, delegate=self)
-            self.devices[uuid_string] = device
-
-        device._update(advertisementData)
-        device._update_rssi(RSSI)
-
         for callback in self.callbacks.values():
             if callback:
                 callback(peripheral, advertisementData, RSSI)
 
         logger.debug(
-            "Discovered device {}: {} @ RSSI: {} (kCBAdvData {}) and Central: {}".format(
-                uuid_string, device.name, RSSI, advertisementData.keys(), central
-            )
+            "Discovered device %s: %s @ RSSI: %d (kCBAdvData %r) and Central: %r",
+            uuid_string,
+            peripheral.name(),
+            RSSI,
+            advertisementData.keys(),
+            central,
         )
 
     def centralManager_didDiscoverPeripheral_advertisementData_RSSI_(
