@@ -4,18 +4,13 @@ import asyncio
 import logging
 import sys
 import warnings
-from typing import List, Optional
+from typing import List, Literal, Optional
 
 if sys.version_info < (3, 11):
     from async_timeout import timeout as async_timeout
 else:
     from asyncio import timeout as async_timeout
 
-if sys.version_info[:2] < (3, 8):
-    from typing_extensions import Literal
-else:
-    from typing import Literal
-
 from android.broadcast import BroadcastReceiver
 from android.permissions import Permission, request_permissions
 from jnius import cast, java_method
@@ -238,6 +233,9 @@ class BleakScannerP4Android(BaseBleakScanner):
         if service_uuids is not None:
             service_uuids = [service_uuid.toString() for service_uuid in service_uuids]
 
+        if not self.is_allowed_uuid(service_uuids):
+            return
+
         manufacturer_data = record.getManufacturerSpecificData()
         manufacturer_data = {
             manufacturer_data.keyAt(index): bytes(manufacturer_data.valueAt(index))
@@ -248,10 +246,10 @@ class BleakScannerP4Android(BaseBleakScanner):
             entry.getKey().toString(): bytes(entry.getValue())
             for entry in record.getServiceData().entrySet()
         }
-        tx_power = result.getTxPower()
+        tx_power = record.getTxPowerLevel()
 
         # change "not present" value to None to match other backends
-        if tx_power == defs.ScanResult.TX_POWER_NOT_PRESENT:
+        if tx_power == -2147483648:  # Integer#MIN_VALUE
             tx_power = None
 
         advertisement = AdvertisementData(
@@ -271,10 +269,7 @@ class BleakScannerP4Android(BaseBleakScanner):
             advertisement,
         )
 
-        if not self._callback:
-            return
-
-        self._callback(device, advertisement)
+        self.call_detection_callbacks(device, advertisement)
 
 
 class _PythonScanCallback(utils.AsyncJavaCallbacks):
