@@ -5,12 +5,14 @@ import os
 import platform
 from typing import (
     Any,
-    Awaitable,
     Callable,
+    Coroutine,
     Dict,
+    Hashable,
     List,
     NamedTuple,
     Optional,
+    Set,
     Tuple,
     Type,
 )
@@ -18,6 +20,9 @@ from typing import (
 from ..exc import BleakError
 from .device import BLEDevice
 
+# prevent tasks from being garbage collected
+_background_tasks: Set[asyncio.Task] = set()
+
 
 class AdvertisementData(NamedTuple):
     """
@@ -50,16 +55,16 @@ class AdvertisementData(NamedTuple):
 
     tx_power: Optional[int]
     """
-    Tx Power data from the received advertising data or ``None`` if not present.
+    TX Power Level of the remote device from the received advertising data or ``None`` if not present.
 
-    .. versionadded:: 0.17.0
+    .. versionadded:: 0.17
     """
 
     rssi: int
     """
     The Radio Receive Signal Strength (RSSI) in dBm.
 
-    .. versionadded:: 0.19.0
+    .. versionadded:: 0.19
     """
 
     platform_data: Tuple
@@ -87,7 +92,7 @@ class AdvertisementData(NamedTuple):
 
 AdvertisementDataCallback = Callable[
     [BLEDevice, AdvertisementData],
-    Optional[Awaitable[None]],
+    Optional[Coroutine[Any, Any, None]],
 ]
 """
 Type alias for callback called when advertisement data is received.
@@ -130,8 +135,17 @@ class BaseBleakScanner(abc.ABC):
         service_uuids: Optional[List[str]],
     ):
         super(BaseBleakScanner, self).__init__()
-        self._callback: Optional[AdvertisementDataCallback] = None
-        self.register_detection_callback(detection_callback)
+
+        self._ad_callbacks: Dict[
+            Hashable, Callable[[BLEDevice, AdvertisementData], None]
+        ] = {}
+        """
+        List of callbacks to call when an advertisement is received.
+        """
+
+        if detection_callback is not None:
+            self.register_detection_callback(detection_callback)
+
         self._service_uuids: Optional[List[str]] = (
             [u.lower() for u in service_uuids] if service_uuids is not None else None
         )
@@ -140,11 +154,10 @@ class BaseBleakScanner(abc.ABC):
 
     def register_detection_callback(
         self, callback: Optional[AdvertisementDataCallback]
-    ) -> None:
-        """Register a callback that is called when a device is discovered or has a property changed.
-
-        If another callback has already been registered, it will be replaced with ``callback``.
-        ``None`` can be used to remove the current callback.
+    ) -> Callable[[], None]:
+        """
+        Register a callback that is called when an advertisement event from the
+        OS is received.
 
         The ``callback`` is a function or coroutine that takes two arguments: :class:`BLEDevice`
         and :class:`AdvertisementData`.
@@ -152,25 +165,86 @@ class BaseBleakScanner(abc.ABC):
         Args:
             callback: A function, coroutine or ``None``.
 
+        Returns:
+            A method that can be called to unregister the callback.
         """
-        if callback is not None:
-            error_text = "callback must be callable with 2 parameters"
-            if not callable(callback):
-                raise TypeError(error_text)
+        error_text = "callback must be callable with 2 parameters"
 
-            handler_signature = inspect.signature(callback)
-            if len(handler_signature.parameters) != 2:
-                raise TypeError(error_text)
+        if not callable(callback):
+            raise TypeError(error_text)
+
+        handler_signature = inspect.signature(callback)
+
+        if len(handler_signature.parameters) != 2:
+            raise TypeError(error_text)
 
         if inspect.iscoroutinefunction(callback):
 
-            def detection_callback(s, d):
-                asyncio.ensure_future(callback(s, d))
+            def detection_callback(s: BLEDevice, d: AdvertisementData) -> None:
+                task = asyncio.create_task(callback(s, d))
+                _background_tasks.add(task)
+                task.add_done_callback(_background_tasks.discard)
 
         else:
             detection_callback = callback
 
-        self._callback = detection_callback
+        token = object()
+
+        self._ad_callbacks[token] = detection_callback
+
+        def remove() -> None:
+            self._ad_callbacks.pop(token, None)
+
+        return remove
+
+    def is_allowed_uuid(self, service_uuids: Optional[List[str]]) -> bool:
+        """
+        Check if the advertisement data contains any of the service UUIDs
+        matching the filter. If no filter is set, this will always return
+        ``True``.
+
+        Args:
+            service_uuids: The service UUIDs from the advertisement data.
+
+        Returns:
+            ``True`` if the advertisement data should be allowed or ``False``
+             if the advertisement data should be filtered out.
+        """
+        # Backends will make best effort to filter out advertisements that
+        # don't match the service UUIDs, but if other apps are scanning at the
+        # same time or something like that, we may still receive advertisements
+        # that don't match. So we need to do more filtering here to get the
+        # expected behavior.
+
+        if not self._service_uuids:
+            # if there is no filter, everything is allowed
+            return True
+
+        if not service_uuids:
+            # if there is a filter the advertisement data doesn't contain any
+            # service UUIDs, filter it out
+            return False
+
+        for uuid in service_uuids:
+            if uuid in self._service_uuids:
+                # match was found, keep this advertisement
+                return True
+
+        # there were no matching service uuids, filter this one out
+        return False
+
+    def call_detection_callbacks(
+        self, device: BLEDevice, advertisement_data: AdvertisementData
+    ) -> None:
+        """
+        Calls all registered detection callbacks.
+
+        Backend implementations should call this method when an advertisement
+        event is received from the OS.
+        """
+
+        for callback in self._ad_callbacks.values():
+            callback(device, advertisement_data)
 
     def create_or_update_device(
         self, address: str, name: str, details: Any, adv: AdvertisementData
